"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const spawn = require('cross-spawn');
const createPkgId_1 = require('./createPkgId');
const debug_1 = require('../debug');
const debug = debug_1.default('pnpm:git');
function resolveGithub(parsedSpec, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const parts = parsedSpec.spec.split('#');
        const repo = parts[0];
        const ref = parts[1] || 'master';
        return {
            id: parsedSpec.spec.replace(/\//g, createPkgId_1.delimiter),
            fetch: (target, opts) => {
                return clone(repo, ref, target);
            }
        };
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = resolveGithub;
function prefixGitArgs() {
    return process.platform === 'win32' ? ['-c', 'core.longpaths=true'] : [];
}
function spawnGit(args) {
    debug(`executing git with args ${args}`);
    const fullArgs = prefixGitArgs().concat(args || []);
    return spawn('git', fullArgs);
}
/**
 * clone a git repository.
 */
function clone(repo, ref, dest) {
    return new Promise((resolve, reject) => {
        const args = ['clone', '-b', ref, repo, dest, '--single-branch'];
        debug(`cloning git repository from ${repo}`);
        const git = spawnGit(args);
        let errMsg = '';
        git.stderr.on('data', (data) => errMsg += data);
        git.on('close', (code) => (code ? errorHandler() : resolve()));
        function errorHandler() {
            debug(`failed to clone repository from ${repo}`);
            reject(new Error(`failed to clone repository from ${repo}
        ${errMsg}`));
        }
    });
}
exports.clone = clone;
//# sourceMappingURL=git.js.map